use std::ptr::null_mut;
use std::sync::atomic::{AtomicPtr, Ordering};
use winapi::ctypes::c_int;
use winapi::shared::minwindef::{DWORD, HINSTANCE, LPARAM, LRESULT, WPARAM};
use winapi::shared::windef::HHOOK__;
use winapi::um::errhandlingapi::GetLastError;
use winapi::um::winuser::{SetWindowsHookExW, UnhookWindowsHookEx};
pub mod keyboard;
pub mod mouse;

pub type HookCallback =
  unsafe extern "system" fn(code: c_int, param: WPARAM, lpdata: LPARAM) -> LRESULT;

pub enum HookError {
  Mouse(DWORD),
  Keyboard(DWORD),
}

pub unsafe fn hook(
  hook_id: c_int,
  hook_ptr: &AtomicPtr<HHOOK__>,
  callback: HookCallback,
) -> Result<(), DWORD> {
  hook_ptr.store(
    SetWindowsHookExW(hook_id, Some(callback), 0 as HINSTANCE, 0),
    Ordering::Relaxed,
  );

  if hook_ptr.load(Ordering::Relaxed).is_null() {
    let error = GetLastError();
    return Err(error);
  }

  Ok(())
}

pub unsafe fn unhook(hook_ptr: &AtomicPtr<HHOOK__>) -> Result<(), DWORD> {
  if !hook_ptr.load(Ordering::Relaxed).is_null() {
    UnhookWindowsHookEx(hook_ptr.load(Ordering::Relaxed));
    hook_ptr.store(null_mut(), Ordering::Relaxed);
  }
  Ok(())
}
